import { S3Client, PutObjectCommand } from '@aws-sdk/client-s3';
import sharp from 'sharp';
import { logger } from '../config/logger';
import { imageResolutions } from '../config/imageResolutions';

const s3Client = new S3Client({
  region: process.env.AWS_REGION || 'ru-central1',
  endpoint: process.env.AWS_ENDPOINT || '',
  credentials: {
    accessKeyId: process.env.AWS_ACCESS_KEY_ID || '',
    secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY || '',
  },
});

// Создание клиента S3
export const createS3Client = () => {
  return new S3Client({
    region: process.env.AWS_REGION || 'ru-central1',
    endpoint: process.env.AWS_ENDPOINT || '',
    credentials: {
      accessKeyId: process.env.AWS_ACCESS_KEY_ID || '',
      secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY || '',
    },
  });
};

const BUCKET_NAME = process.env.AWS_S3_BUCKET || '';

export const s3Service = {
  uploadOriginalFile: async (file: Express.MulterS3.File) => {
    logger.info(`Оригинальный файл загружен в S3: ${file.key}`);
    return file.key;
  },

  optimizeAndUpload: async (fileBuffer: Buffer, originalKey: string, resolutionId: string) => {
    const selectedResolution = imageResolutions.find(r => r.id === resolutionId);
    if (!selectedResolution) {
      throw new Error('Недопустимое разрешение');
    }

    try {
      // Оптимизация изображения
      const optimizedBuffer = await sharp(fileBuffer)
          .resize(selectedResolution.width, selectedResolution.height, {
            fit: 'inside',
            withoutEnlargement: true,
          })
          .webp({ quality: 80 })
          .toBuffer();

      // Генерация нового ключа для оптимизированного файла
      const optimizedKey = originalKey.replace(/\.[^/.]+$/, '.webp');

      // Загрузка оптимизированного файла
      await s3Client.send(
          new PutObjectCommand({
            Bucket: BUCKET_NAME,
            Key: optimizedKey,
            Body: optimizedBuffer,
            ContentType: 'image/webp',
          })
      );

      logger.info(`Оптимизированное изображение загружено в S3: ${optimizedKey}`);

      return {
        key: optimizedKey,
        width: selectedResolution.width,
        height: selectedResolution.height,
        format: 'webp',
        size: optimizedBuffer.length,
      };
    } catch (error) {
      logger.error('Ошибка при оптимизации и загрузке файла:', error);
      throw error;
    }
  },
};
